#
# tkform.tcl: Tcl/Tk form description
#
# Original: Mar 29 1991
# $Id: tkform.tcl,v 1.2 1993/08/18 11:24:45 utashiro Exp $
#
proc tkform {path labelarray form {context -1}} {

    if {$context == -1} {set context [info level]; incr context -1}
    upvar #$context $labelarray larray
    upvar #$context debugform debug
    upvar #$context self self; set self $path
    upvar option poption

    if {[info exists debug] && $debug} {puts "making $path"}

    #
    # If the first element is frame or toplevel, make it and pack folowing
    # widgets on it.  If the second element start by -, it is treated as
    # an option for the first element.  If the first one is one of top,
    # bottom, left and right, frame is assumed for shortcut.
    #
    case [set command [lindex $form 0]] in {
	"frame top bottom right left" {
	    set start 0
	    set eval "frame $path"
	    if [info exist poption(frame)] {
		append eval " $poption(frame)"
	    }
	    if {$command == "frame"} {set start 1}
	    if {[string match -* [lindex $form $start]]} {
		incr start
		append eval " [lindex $form 1]"
	    }
	    set thisframe [uplevel #$context $eval]
	    if {$start} {set form [lrange $form $start end]}
	}
	toplevel {
	    set eval "toplevel $path"
	    if {[string match -* [lindex $form 1]]} {
		append eval " [lindex $form 1]"
		set form [lrange $form 1 end]
	    }
	    set thisframe [uplevel #$context $eval]
	    set form [lrange $form 1 end]
	}
	default {
	    set eval [concat [lindex $form 0] $path]
	    if [info exist poption($command)] {
		set eval [concat $eval $poption($command)]
	    }
	    set eval [concat $eval [lreplace $form 0 0]]
	    return [uplevel #$context $eval]
	}
    }
      
    #
    # inherit option array
    #
    if [info exist poption] {
	foreach k [array names poption] {
	    set option($k) $poption($k)
	}
    }

    #
    # Initialize pack options
    #
    set place	top
    set padx	0
    set pady	0
    set fill	{}
    set expand	{}
    set frame	center
    set prev	{}
    if [info exist option(pack)] {
	set form [concat $option(pack) $form]
    }

    #
    # Initialize label vars
    #
    set label {}; set from {}; set to {}

    set seq 0
    foreach x $form {
	#
	# Check pack options
	#
	if {$prev != ""} {
	    case $prev in {
		padx {set padx $x}
		pady {set pady $x}
		frame {set frame $x}
		configure {
		    set option([lindex $x 0]) [lrange $x 1 end]
		}
	    }
	    set prev {}
	    continue
	}
	case $x in {
	    "top bottom left right" {
		set place $x
		continue
	    }
	    nofill {set fill {}; continue}
	    "fill fillx filly" {
		set fill $x
		continue
	    }
	    expand {set expand $x; continue}
	    noexpand {set expand {}; continue}
	    "padx pady frame configure" {
		set prev $x
		continue
	    }
	}

	#
	# Label ( label: { ...} )
	#
	if {[regexp {(.*):$} $x matched label]} {
	    regexp {(.*)\[([^\-]+)-([^\-]+)\]$} $label matched label from to
	    if [regexp {(.*)\[([^\-]+)\]} $label matched label to] {
		set from 1
	    }
	    if [regexp {[^0-9]} $from] {
		set from [uplevel #$context "expr $from"]
	    }
	    if [regexp {[^0-9]} $to] {
		set to [uplevel #$context "expr $to"]
	    }
	    continue
	}

	#
	# OK, now make a new object!
	#
	set packopt "$place $fill $expand padx $padx pady $pady frame $frame"
	if {$label == "prog"} {
	    uplevel #$context $x;
	} elseif {$label != {} && $from != {} && $to != {}} {
	    upvar #$context $label i
	    for {set i $from} {$i <= $to} {incr i} {
		incr seq
		set newpath $path.$label$i
		tkform $newpath $labelarray $x $context
		pack append $thisframe $newpath $packopt
		set larray($label$i) $newpath
	    }
	} else {
	    set newpath $path.[incr seq]
	    if {$label != ""} {	# If a label is defined, remember it.
		set larray($label) [set newpath $path.$label]
	    }
	    tkform $newpath $labelarray $x $context
	    pack append $thisframe $newpath $packopt
	}
	set label {}; set from {}; set to {}
    }

    return $thisframe
}
